<!DOCTYPE html>
<html>
  <head>
    <meta charset="utf-8" />
    <title>Reset password link was sent</title>
  </head>
  <body>
    <p>
      You can reset your password by clicking
      <a href="{{ verification_url }}">here</a>.
    </p>
  </body>
</html>
